// Complex.cpp
//
// ICS 65 Fall 2012
// Code Example

#include "Complex.h"


// You'll notice that I haven't written any of these operators in terms
// of the others.  For example, I opted not to write the != operator to
// simply return the opposite of a call to the == operator.  You could
// do it that way, but it would potentially limit performance.

Complex& Complex::operator+=(const Complex& d)
{
	realPart += d.realPart;
	imaginaryPart += d.imaginaryPart;
	return *this;
}


Complex operator+(const Complex& c, const Complex& d)
{
	return Complex(c.real() + d.real(), c.imaginary() + d.imaginary());
}


Complex& Complex::operator-=(const Complex& d)
{
	realPart -= d.realPart;
	imaginaryPart -= d.imaginaryPart;
	return *this;
}


Complex operator-(const Complex& c, const Complex& d)
{
	return Complex(c.real() - d.real(), c.imaginary() - d.imaginary());
}


bool operator==(const Complex& c, const Complex& d)
{
	return c.real() == d.real() && c.imaginary() == d.imaginary();
}


bool operator!=(const Complex& c, const Complex& d)
{
	return c.real() != d.real() || c.imaginary() != d.imaginary();
}


std::ostream& operator<<(std::ostream& out, const Complex& c)
{
	// Here, I'll write the parts of a Complex to whatever output stream
	// I've been given.  We'll write it in one of these formats (depending
	// on the value of the imaginary part):
	//
	//     3.5+2.5i
	//     3.5-2.5i
	//     3.5

	out << c.real();

	if (c.imaginary() < 0.0)
	{
		out << "-" << -c.imaginary() << "i";
	}
	else if (c.imaginary() > 0.0)
	{
		out << "+" << c.imaginary() << "i";
	}

	// The << operator on a stream must return the stream as its return
	// value.  This is because of the way we string together uses of the
	// << operator to write to streams.  For example:
	//
	//    cout << "Hello" << " World!";
	//
	// is processed from left to right, as though it were written:
	//
	//    (cout << "Hello") << " World!";
	// 
	// In order for the second << operator to have a stream to write its
	// output to, the first one must have returned the stream.  So, we'll
	// simply say this:
	return out;
}


std::istream& operator>>(std::istream& in, Complex& c)
{
	// It's usually a good idea for the >> operator to be the analogue of
	// the << operator.  In other words, it's usually good for the >>
	// operator to be able to successfully read the output generated by
	// the << operator.  In a fit of extreme laziness, I'll skip that
	// requirement and simply read two doubles, but I'll feel badly about
	// this until the day I die.

	double real;
	double imaginary;

	in >> real >> imaginary;

	c.real() = real;
	c.imaginary() = imaginary;


	// When we're finished, we return the stream, for similar reasons to
	// why did it in the << operator: because we want to be able to string
	// together >> operators into one long expression.
	return in;
}
